/*
			   Login Watcher 

                  written by tHE rECIdjVO [fTR]
                      <recidjvo@pkcrew.org>


This program waits for some user to login or logout the system,
showing a warning message on the tty from which it was ran on.

The check is performed every INTERVAL seconds, looking for info
in the utmp file: so, if anyone (root) modify the entries, he
is hidden to us too.

Compile with: % gcc -o lw lw.c

Usage:	lw -l LOGIN_USER
								*/

/*              -=| START OF CONFIGURATION SECTION |=-          */

#define UTMPFILE "/var/run/utmp"
#define INTERVAL 5

/*              -=| END OF CONFIGURATION SECTION |=-            */

// Const declaration
#define PROGRAM "lw"
#define INFO "by tHE rECIdjVO"
#define CONTACT "<recidjvo@pkcrew.org>"
#define BOLD "\033[1m"
#define FLAT "\033[m"
#define BLINK "\033[5m"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>
#include <utmp.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

// Functions prototypes
int main(int argc, char *argv[]);
void usage(void);
int chekname(char *login);
void lwatch(char *login);

int main(int argc, char *argv[])
{
	printf("\n   " BOLD PROGRAM FLAT " - " INFO " " CONTACT "\n\n");
	if ((argc != 3) || (strcmp(argv[1], "-l") != 0)) {
		usage();
		exit(1);
	}

	if (checkname(argv[2]) == 0) {
		printf(PROGRAM ": %s is not a valid user.\n", argv[2]);
		exit(1);
	}

	lwatch(argv[2]);
	exit(0);
}

void usage(void)
{

        // Just print out usage

        printf("   usage:   " PROGRAM " -l LOGIN_USER\n\n");
	return;
}

int checkname(char *login)
{
	struct passwd *userdata;

	// Check if it's a valid login name
	if ((userdata = getpwnam(login)) == NULL) {
		return(0);
	}
	printf("User selected for watching: " BOLD "%s" FLAT " (%s)\n\n", login, userdata->pw_gecos);
	return(1);
}

void lwatch(char *login)
{
	int mypid, f, on = 0, count = 0;
	struct utmp utmpent;
	char tty[8];
	
	if ((f = open(UTMPFILE, O_RDONLY)) < 0) {
		printf(PROGRAM ": unable to read " UTMPFILE ".\n");
		exit(1);
	}

	while(read(f, &utmpent, sizeof(utmpent)) > 0) {
		if (strcmp(utmpent.ut_name, login) == 0) {
			on = 1;
			printf(BOLD "[%s] is on the system now on %s" FLAT ".\n\n", login, utmpent.ut_line);
			break;
		}
	}

	close(f);

	printf("Going in the background...\n\n");
	if ((mypid = fork()) != 0) {
		exit(0);
	}

	while(1) {
		sleep(INTERVAL);
	        if ((f = open(UTMPFILE, O_RDONLY)) < 0) {
	                printf(PROGRAM ": unable to read " UTMPFILE ".\n");
	                exit(1);
	        }

		count = 0;
	        while(read(f, &utmpent, sizeof(utmpent)) > 0) {
	                if (strcmp(utmpent.ut_name, login) == 0) {
				count++;
				bzero(tty, 8);				
				strcpy(tty, utmpent.ut_line);
			}		
	        }
	        close(f);
		
		if ((on) && (!count)) {
			on = 0;
 			printf("\n\n" BLINK "\tALERT!!!\tALERT!!!\tALERT!!!\tALERT!!!" FLAT "\n");
			printf(BOLD "[%s] has left the system now" FLAT ".", login);
 			printf("\n" BLINK "\tALERT!!!\tALERT!!!\tALERT!!!\tALERT!!!" FLAT "\n\n");
		} else if ((!on) && (count > 0)) {
                        on = 1;
 			printf("\n\n" BLINK "\tALERT!!!\tALERT!!!\tALERT!!!\tALERT!!!" FLAT "\n");
                        printf(BOLD "[%s] has login now on %s" FLAT ".", login, tty);
 			printf("\n" BLINK "\tALERT!!!\tALERT!!!\tALERT!!!\tALERT!!!" FLAT "\n\n");
		}
	}
}

/*				EOF lw.c			*/
